unit Im;

interface

uses
  Graphics, SysUtils, Classes, Controls,
  ExtCtrls, Windows, Legend, Clipbrd;

type
{$R *.res}
  TOptions=record
    drawPointNumbers,
    drawTriangles,
    drawPoints,
    drawIsolines,
    labelIsolines,
    drawMinMaxValues,
    drawMinMax,
    fill,
    standardColors: boolean;
    revers: boolean;
    showTime: boolean;
    gaps: boolean;
    nLines: integer;
    delta1: integer;
    colors: array[1..100]of Longint;
  end;

  TIsolineView=(points,
                points_lines,
                lines_triangles,
                triangles,
                lines,
                polygons);

  TIm = class(TImage)
  private
    FLegend: TLegend;
    hndl: integer;
    DLLOK: boolean;
    Ffname: string;
    options: TOptions;
    Fview: TIsolineView;
    Fnlines: integer;

    LegendBmpToClp: procedure; stdcall;

    BuildIsolinesMultiClp: procedure
      (fname: string;
       n: integer;
       opt: TOptions;
       w,h: integer;
       legendW,legendH: integer); stdcall;

    GetIsolineOptions: function: TOptions; stdcall;
    procedure InitOpt;

  protected
  public
    constructor Create(aowner: TComponent); override;
    destructor Destroy; override;
    function Execute: boolean;
  published
    property fname: string read Ffname write Ffname;
    property legend: TLegend read FLegend write FLegend;
    property view: TIsolineView read Fview write Fview;
    property nLines: integer read Fnlines write fnlines;
  end;

procedure Register;

implementation

function TIm.Execute: boolean;
var
  bmp1,legendBmp: Graphics.TBitmap;
begin
  Result:=false;
  if DLLOK and (fname<>'') then
  begin
    options:=GetIsolineOptions;
    InitOpt;

    options.nLines:=nLines;
    with options do begin
    case view of
      points:
        begin
          drawPointNumbers:=true;
          drawTriangles:=false;
          drawPoints:=true;
          drawIsolines:=false;
          fill:=false;
        end;
      points_lines:
        begin
          drawPointNumbers:=false;
          drawTriangles:=false;
          drawPoints:=true;
          drawIsolines:=true;
          fill:=false;
        end;
      lines_triangles:
        begin
          drawPointNumbers:=false;
          drawTriangles:=true;
          drawPoints:=false;
          drawIsolines:=true;
          fill:=false;
        end;
      triangles:
        begin
          drawPointNumbers:=false;
          drawTriangles:=true;
          drawPoints:=false;
          drawIsolines:=false;
          fill:=false;
        end;
      lines:
        begin
          drawPointNumbers:=false;
          drawTriangles:=false;
          drawPoints:=false;
          drawIsolines:=true;
          fill:=false;
        end;
      polygons:
        begin
          drawPointNumbers:=false;
          drawTriangles:=false;
          drawPoints:=false;
          drawIsolines:=true;
          fill:=true;
        end;
    end;
    end;

    bmp1:=Graphics.TBitmap.Create;
    bmp1.Width:=Width;
    bmp1.Height:=Height;

    legendBmp:=Graphics.TBitmap.Create;
    if legend<>nil then
    begin
      legendBmp.Width:=legend.Width;
      legendBmp.Height:=legend.Height;
    end
    else
    begin
      legendBmp.Width:=300;
      legendBmp.Height:=400;
    end;

    Clipboard.Clear;
    BuildIsolinesMultiClp(fname,1,options,bmp1.Width,bmp1.Height,
                        legendBmp.Width,legendBmp.Height);

    bmp1.Assign(Clipboard);
    Canvas.Draw(0,0,bmp1);

    if legend<>nil then
    begin
      LegendBmpToClp;
      legendBmp.Assign(Clipboard);
      legend.Canvas.Draw(0,0,legendBmp);
    end;

    Result:=true;
  end;
end;

destructor TIm.Destroy;
begin
  inherited Destroy;
  FreeLibrary(hndl);
end;

procedure TIm.InitOpt;
begin
  options.drawPointNumbers:=false;
  options.drawTriangles:=false;
  options.drawPoints:=false;
  options.nLines:=5;
  options.drawIsolines:=true;
  options.drawMinMaxValues:=true;
  options.drawMinMax:=true;
  options.fill:=true;
  options.revers:=true;
  options.showTime:=false;
  options.gaps:=true;
  options.delta1:=10;
  options.labelIsolines:=false;
  options.standardColors:=true;
end;

constructor TIm.Create(aowner: TComponent);
begin
  inherited Create(aowner);
  view:=polygons;
  nLines:=5;
  DLLOK:=false;
  hndl:=LoadLibrary('isolines.dll');
  if hndl<>0 then
    begin
      BuildIsolinesMultiClp:=GetProcAddress(hndl,'BuildIsolinesMultiClp');
      LegendBmpToClp:=GetProcAddress(hndl,'LegendBmpToClp');

      GetIsolineOptions:=GetProcAddress(hndl,'GetIsolineOptions');
      if (@LegendBmpToClp <> nil)
         and (@BuildIsolinesMultiClp <> nil)
         and (@GetIsolineOptions <> nil)
      then
      begin
        DLLOK:=true;
      end
    end;
end;

procedure Register;
begin
  RegisterComponents('Isolines', [TIm]);
end;

end.
